<!doctype html>
<html>
<link rel="stylesheet" href="style.css" />
<title></title>
<body>
<div class="container">
    <div class="sixteen columns">
      {{ content }}
    </div>
</div>
</body>
</html>
